// Class QLookAndFeel 
// Defines Look and Feel. Colors, Textures, default Font size and so on.
// Filename QLookAndFeel.cpp
// FaPra 2003-2004, A5,  Alexander Kramer


#include "QLookAndFeel.h"
#include "tga.h"
#include <GL/glut.h>
#include <iostream>
using namespace std;

// ----------------------------------------------------------

//! draws bevel from x,y-x+w,y+h. State : up,down,disabled
void QLookAndFeel::drawBevel( float  x , float  y , float  w , float  h , QBevelState  state )
{
 glBegin(GL_LINES);
  switch (state)
  {
    case bsUp: BevelRaisedColor->setAsCurrent();
    break;
    case bsDown:BevelLoweredColor->setAsCurrent();
    break;
    case bsDisabled : BevelDisabledColor->setAsCurrent();
    break;
  }


  glVertex2f(x,y+h);
  glVertex2f(x,y);

  glVertex2f(x,y);
  glVertex2f(x+w,y);

  switch (state)
  {
    case bsDown: BevelRaisedColor->setAsCurrent();
    break;
    case bsUp: BevelLoweredColor->setAsCurrent();
    break;
    case bsDisabled : BevelDisabledColor->setAsCurrent();
    break;
  }

  glVertex2f(x+w,y);
  glVertex2f(x+w,y+h);

  glVertex2f(x+w,y+h);
  glVertex2f(x,y+h);

  glEnd();
} // end of drawBevel()

//! draws bevel from x,y-x+w,y+h. State : up,down,disabled
void QLookAndFeel::drawRect( float  x , float  y , float  w , float  h )
{
  glVertex2f(x,y+h);
  glVertex2f(x,y);
  glVertex2f(x+w,y);
  glVertex2f(x+w,y+h);
} // end of drawRect()

//! draws given texture with given size
void QLookAndFeel::drawTexture( unsigned  int  texID , int  width , int  height )
{
  glEnable(GL_BLEND);								// For transparent background
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D, texID);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0, 0.0); glVertex2i(0, height);
  glTexCoord2f(0.0, 1.0); glVertex2i(0, 0);
  glTexCoord2f(1.0, 1.0); glVertex2i(width, 0);
  glTexCoord2f(1.0, 0.0); glVertex2i(width, height);
  glEnd();
  glDisable(GL_TEXTURE_2D);

  glDisable(GL_BLEND);
} // end of drawTexture()

//! draws checkbox texture
void QLookAndFeel::drawCheckBox( bool  checked )
{
  BackGroundColor->setAsCurrent();
  if (checked)
    drawTexture(CheckBoxCheckedTexID,16,16);
  else
    drawTexture(CheckBoxTexID,16,16);
} // end of drawCheckBox()

//! draws radio button texture
void QLookAndFeel::drawRadioButton( bool  checked )
{
  BackGroundColor->setAsCurrent();
  if (checked)
  {
    drawTexture(RadioButtonCheckedTexID,16,16);
  }
  else
  {
    drawTexture(RadioButtonTexID,16,16);
  }
} // end of drawRadioButton()

//! draws close button texture
void QLookAndFeel::drawCloseButton( bool  down , int  width , int  height )
{
  if (down)
    drawTexture(CloseButtonDownTexID,width,height);
  else
    drawTexture(CloseButtonTexID,width,height);
} // end of drawCloseButton()

//! draws maximize button texture
void QLookAndFeel::drawMaximizeButton( bool  down , int  width , int  height )
{
 if (down)
    drawTexture(MaximizeButtonDownTexID,width,height);
  else
    drawTexture(MaximizeButtonTexID,width,height);
} // end of drawMaximizeButton()

//! initialization code
void QLookAndFeel::Init()
{

  Font->getColor()->set(0,0,0,1);
  Font->setSize(16);
  BackGroundColor->set(0.8,0.8,0.8,1);
  BevelLoweredColor->set(0.5,0.5,0.5,1);
  BevelRaisedColor->set(1,1,1,1);
  BevelDisabledColor->set(0.7,0.7,0.7,1);
  HighlightColor->set(198.0/255.0,245.0/255.0,1.0,1.0);
  DisabledColor->set(0.5,0.5,0.5,1.0);

  CheckBoxTexID           = createTexture("checkbox.tga");
  CheckBoxCheckedTexID    = createTexture("checkboxchecked.tga");
  RadioButtonTexID        = createTexture("radio.tga");
  RadioButtonCheckedTexID = createTexture("radiochecked.tga");
  CloseButtonTexID        = createTexture("closebutton.tga");
  CloseButtonDownTexID    = createTexture("closebutton_down.tga");
  MaximizeButtonTexID     = createTexture("maximizebutton.tga");
  MaximizeButtonDownTexID = createTexture("maximizebutton_down.tga");
} // end of Init()

//! clean up code
void QLookAndFeel::CleanUp()
{
  glDeleteTextures(1,&CheckBoxTexID);
  glDeleteTextures(1,&CheckBoxCheckedTexID);
  glDeleteTextures(1,&RadioButtonTexID);
  glDeleteTextures(1,&RadioButtonCheckedTexID);
  glDeleteTextures(1,&CloseButtonTexID);
  glDeleteTextures(1,&CloseButtonDownTexID);
  glDeleteTextures(1,&MaximizeButtonTexID);
  glDeleteTextures(1,&MaximizeButtonDownTexID);

} // end of CleanUp()

//! getter method for BackGroundColor
QColor* QLookAndFeel::getBackGroundColor()
{
return BackGroundColor;
} // end of getBackGroundColor()

//! getter method for DisabledColor
QColor* QLookAndFeel::getDisabledColor()
{
return DisabledColor;
} // end of getDisabledColor()

//! getter method for BevelRaisedColor
QColor* QLookAndFeel::getBevelRaisedColor()
{
return BevelRaisedColor;
} // end of getBevelRaisedColor()

//! getter method for BevelLoweredColor
QColor* QLookAndFeel::getBevelLoweredColor()
{
return BevelLoweredColor;
} // end of getBevelLoweredColor()

//! getter method for BevelDisabledColor
QColor* QLookAndFeel::getBevelDisabledColor()
{
return BevelDisabledColor;
} // end of getBevelDisabledColor()

//! getter method for HighlightColor
QColor* QLookAndFeel::getHighlightColor()
{
return HighlightColor;
} // end of getHighlightColor()

//! getter method for Font
QFont* QLookAndFeel::getFont()
{
return Font;
} // end of getFont()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! prints itself to the cout
void QLookAndFeel::print()
{
cout << "Class : QLookAndFeel" << endl;
} // end of print()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Constructor
 QLookAndFeel::QLookAndFeel(  )
{
CheckBoxTexID = 0;
CheckBoxCheckedTexID = 0;
RadioButtonCheckedTexID = 0;
RadioButtonTexID = 0;
CloseButtonTexID = 0;
CloseButtonDownTexID = 0;
MaximizeButtonTexID = 0;
MaximizeButtonDownTexID = 0;
RestoreButtonTexID = 0;
RestoreButtonDownTexID = 0;
MinimizeButtonTexID = 0;
MinimizeButtonDownTexID = 0;
BackGroundColor = new QColor;
DisabledColor = new QColor;
BevelRaisedColor = new QColor;
BevelLoweredColor = new QColor;
BevelDisabledColor = new QColor;
HighlightColor = new QColor;
Font = new QFont;
Init();
} // end of QLookAndFeel()

// WARNING : The implementation of this method will be automatically generated with code generator.
// To prevent this add '//KEEP' at the first line of the implementation.(first line after '{')
//! Destructor
 QLookAndFeel::~QLookAndFeel()
{
delete Font;
delete HighlightColor;
delete BevelDisabledColor;
delete BevelLoweredColor;
delete BevelRaisedColor;
delete DisabledColor;
delete BackGroundColor;
CleanUp();
} // end of ~QLookAndFeel()

// end of implementation of QLookAndFeel


