// Class QWidget 
// Filename QWidget.h 
// FaPra 2003-2004, A5, Alexander Kramer

#ifndef QWidget_h 
#define QWidget_h 


//! Pre-declaration of QWidget
class QWidget;


#include "QEvent.h"
#include "QLookAndFeel.h"
#include <GL/glut.h>
#include <list>
using namespace std;

//! Common super class for all widgets. Here is also the basic implementation of event passing and dragging
class QWidget
{
private:
//! dragging position x
int DragStartX;

//! dragging position y
int DragStartY;

protected:
//! width of widget
int Width;

//! height of widget
int Height;

//! top offset
int Top;

//! left offset
int Left;

//! minimum width of widget
int MinWidth;

//! minimum height of widget
int MinHeight;

//! left border to client region
int ClientBorderLeft;

//! right border to client region
int ClientBorderRight;

//! top border to client region
int ClientBorderTop;

//! botom border to client region
int ClientBorderBottom;

//! enabled status
bool Enabled;

//! visible status
bool Visible;

//! dragging status
bool Dragging;

//! Look and Feel
QLookAndFeel* LookAndFeel;

//! Owner of this widget
QWidget* Owner;

//! Defines the child widget to which keyboard events should be passed
QWidget* FocusPath;

//! list of children widgets
list<QWidget*> Children;

public:
//! event handler for mouse click
 void (*onClick)(QWidget*,QEvent*);

//! event handler for mouse down
 void (*onMouseDown)(QWidget*,QEvent*);

//! event handler for mouse up
 void (*onMouseUp)(QWidget*,QEvent*);

//! event handler for mouse up
 void (*onMouseMove)(QWidget*,QEvent*);

//! event handler for keyboard
 void (*onKey)(QWidget*,QEvent*);

//! event handler for custom drawings
 void (*onCustomDraw)(QWidget*,QEvent*);

//! event handler for handling the changes of position and size
 void (*onResize)(QWidget*);

//! adds given widget to the children
virtual void add( QWidget*  widget );

//! removes given widget from the children
virtual void remove( QWidget*  widget );

//! sends request for freeing a widget to parent
virtual void freeWidget( QWidget*  widget );

//! clears the focus path by setting it to 0 recursively
 void clearFocusPath();

//! returns true if widget has keyboard focus
 bool getFocused();

//! frees the widget and all resources. This method must be used insted of explicit delete
 void free();

//! primary event receiver method. Returns true, if event was handled. In this case the event won't be passed to next child
 bool event( QEvent*  e );

//! post request for redisplay
virtual void redraw();

//! sends the event to each child. Returns true if event was handled
 bool eventToChildren( QEvent*  e );

//! sends the event to each child in reversed order. Returns true if event was handled
 bool eventToChildrenReversed( QEvent*  e );

//! brings the given widget to the front. Must be a child widget
virtual void bringToFront( QWidget*  widget );

//! Returns true if a given widget is in front
 bool isInFront( QWidget*  widget );

//! handles the keyboard event
 bool handleKeyboardEvent( QEvent*  e );

//! handles the mouse event for this widget
 bool handleMouseEvent( QEvent*  e );

//! handles the dragging
 void handleDragging( QEvent*  e );

//! tries to set focus to the current widget
 void doFocus( QEvent*  e );

//! called if Sender wants to announce new focused control
virtual void requestFocus( QWidget*  Sender , QWidget*  FocusedWidget );

//! changes the global drawing context
 void changeContext();

//! restores the global drawing context
 void restoreContext();

//! calls delete for every child widget
 void freeChildren();

//! draws itself with OpenGL. Here is the OpenGL drawing code
virtual void draw( QEvent*  e );

//! layout the children widgets. This method is called every time the size is changed. Have to be overwritten for respond on size changes
virtual void layout();

//! have to return true if widget should be dragged with mouse. This method is overwritten in sub classes. If returns true, widget will be dragged
virtual bool canDrag( QEvent*  e );

//! processing mouse/keyboard events which are in widgets region. Mouse coordinates are relative to Owner
virtual bool processEvent( QEvent*  e );

//! processing mouse events which are in Owner widget region. Mouse coordinates are relative to Owner
virtual bool processMouseOwner( QEvent*  e );

//! called if a child is added/removed
virtual void childrenChanged();

//! defines whether control can have keyboard focus. Should return true if Widget wants to get keyboard focus
virtual bool canFocus( QEvent*  e );

//! initialization code
virtual void Init();

//! clean up code
virtual void CleanUp();

//! getter method for Width
virtual int getWidth();

//! setter method for Width
virtual void setWidth( int  newValue );

//! getter method for Height
virtual int getHeight();

//! setter method for Height
virtual void setHeight( int  newValue );

//! getter method for Top
virtual int getTop();

//! setter method for Top
virtual void setTop( int  newValue );

//! getter method for Left
virtual int getLeft();

//! setter method for Left
virtual void setLeft( int  newValue );

//! getter method for MinWidth
virtual int getMinWidth();

//! setter method for MinWidth
virtual void setMinWidth( int  newValue );

//! getter method for MinHeight
virtual int getMinHeight();

//! setter method for MinHeight
virtual void setMinHeight( int  newValue );

//! getter method for ClientBorderLeft
virtual int getClientBorderLeft();

//! setter method for ClientBorderLeft
virtual void setClientBorderLeft( int  newValue );

//! getter method for ClientBorderRight
virtual int getClientBorderRight();

//! setter method for ClientBorderRight
virtual void setClientBorderRight( int  newValue );

//! getter method for ClientBorderTop
virtual int getClientBorderTop();

//! setter method for ClientBorderTop
virtual void setClientBorderTop( int  newValue );

//! getter method for ClientBorderBottom
virtual int getClientBorderBottom();

//! setter method for ClientBorderBottom
virtual void setClientBorderBottom( int  newValue );

//! getter method for Enabled
virtual bool getEnabled();

//! setter method for Enabled
virtual void setEnabled( bool  newValue );

//! getter method for Visible
virtual bool getVisible();

//! setter method for Visible
virtual void setVisible( bool  newValue );

//! getter method for LookAndFeel
virtual QLookAndFeel* getLookAndFeel();

//! setter method for LookAndFeel
virtual void setLookAndFeel( QLookAndFeel*  newValue );

//! getter method for Owner
virtual QWidget* getOwner();

//! setter method for Owner
virtual void setOwner( QWidget*  newValue );

//! getter method for FocusPath
virtual QWidget* getFocusPath();

//! setter method for FocusPath
virtual void setFocusPath( QWidget*  newValue );

//! prints itself to the cout
 void print();

//! Constructor
  QWidget( QWidget*  aOwner );

//! Destructor
virtual  ~QWidget();


};
#endif

